package com.docus.webservice.handler;


import com.alibaba.fastjson.JSON;
import com.docus.webservice.dto.SelectColumn;
import com.docus.webservice.entity.MrReportError;
import com.docus.webservice.entity.TBasic;
import com.docus.webservice.enums.Codes;
import com.docus.webservice.service.IMrReportErrorService;
import com.docus.webservice.service.ITBasicService;
import com.docus.webservice.utils.JsonUtils;
import com.docus.webservice.utils.ResultUtils;
import com.docus.webservice.utils.XmlUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;

import javax.jws.WebService;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.IOException;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@WebService
public class TBasicWebService implements ITBasicWebService {

    @Autowired
    private ITBasicService itBasicService;

    @Autowired
    private IMrReportErrorService mrReportErrorService;

    /**
     * 获取jar包所在位置
     *
     * @return
     */
    private String CurrentPath() {
        File dir = new File(".");
        String currentpath = "";
        try {
            currentpath = dir.getCanonicalPath();
        } catch (IOException e) {
            e.printStackTrace();
        }
        return currentpath;
    }

    /**
     * 记录日志
     */
    private void insertLog(String xml) {
        MrReportError mrReportError = new MrReportError();
        mrReportError.setCreateTime(new Date());
        mrReportError.setReportType(Integer.parseInt(Codes.ERROR_CODE.getMessage()));
        mrReportError.setXml(xml);
        mrReportErrorService.save(mrReportError);
    }

    @Override
    public String savaAndSub(String tbasic) {
        XmlUtils xmlUtils;
        try {
            //解析xml
            xmlUtils = new XmlUtils(new ByteArrayInputStream(tbasic.getBytes("UTF-8")));
            //解析json映射文件
            String json = JsonUtils.readJsonFile(CurrentPath() + Codes.JSON_ADDRESS.getMessage());
            Map jsonMap = JSON.parseObject(json, Map.class);
            HashMap<String, Object> tBasicMap = new HashMap<>();
            HashMap<String, Object> tBasicSubMap = new HashMap<>();
            List<SelectColumn> selectColumns = JSON.parseArray(String.valueOf(jsonMap.get(Codes.SELECT_COLUMNS.getMessage())), SelectColumn.class);
            String value;
            for (SelectColumn selectColumn : selectColumns) {
                try {
                    value = xmlUtils.getElementText(selectColumn.getQualityColumn()) == null ? null : String.valueOf(xmlUtils.getElementText(selectColumn.getQualityColumn()));
                } catch (RuntimeException e) {
                    String message = e.getMessage();
                    return ResultUtils.fail(message);
                }
                if (StringUtils.isBlank(value)) {
                    if (selectColumn.getByNull() == 0) {
                        insertLog(tbasic);
                        return ResultUtils.fail(selectColumn.getQualityColumn() + " 字段不能为空值");
                    } else {
                        continue;
                    }
                } else {
                    if (selectColumn.getTableName().equals("t_basic")) {
                        tBasicMap.put(selectColumn.getColumnName(), value);
                    }
                    if (selectColumn.getTableName().equals("t_basic_sub")) {
                        tBasicSubMap.put(selectColumn.getColumnName(), value);
                    }
                }
            }
            //处理业务
            TBasic tBasic;
            if (tBasicMap != null && tBasicMap.size() > 0) {
                tBasic = JSON.parseObject(JSON.toJSONString(tBasicMap), TBasic.class);
                tBasic.setCreateTime(new Date());
                try {
                    itBasicService.savaAndSub(tBasic, tBasicSubMap);
                } catch (RuntimeException e) {
                    insertLog(tbasic);
                    return ResultUtils.fail(e.getMessage());
                }
            } else {
                insertLog(tbasic);
                return ResultUtils.fail("请提供完整的xml");
            }
        } catch (Exception e) {
            e.printStackTrace();
            return ResultUtils.fail();
        }
        return ResultUtils.success().asXML();
    }

    @Override
    public String updateAndSub(String tbasic) {
        XmlUtils xmlUtils;
        try {
            //解析xml
            xmlUtils = new XmlUtils(new ByteArrayInputStream(tbasic.getBytes("UTF-8")));
            //解析json映射文件
            String path = CurrentPath() + Codes.JSON_ADDRESS.getMessage();
            String json = JsonUtils.readJsonFile(path);
            Map jsonMap = JSON.parseObject(json, Map.class);
            HashMap<String, Object> tBasicMap = new HashMap<>();
            HashMap<String, Object> tBasicSubMap = new HashMap<>();
            List<SelectColumn> selectColumns = JSON.parseArray(String.valueOf(jsonMap.get(Codes.SELECT_COLUMNS.getMessage())), SelectColumn.class);
            String value;
            for (SelectColumn selectColumn : selectColumns) {
                try {
                    value = xmlUtils.getElementText(selectColumn.getQualityColumn()) == null ? null : String.valueOf(xmlUtils.getElementText(selectColumn.getQualityColumn()));
                } catch (RuntimeException e) {
                    String message = e.getMessage();
                    return ResultUtils.fail(message);
                }
                if (StringUtils.isBlank(value)) {
                    if (selectColumn.getByNull() == 0) {
                        insertLog(tbasic);
                        return ResultUtils.fail(selectColumn.getQualityColumn() + " 字段不能为空值");
                    } else {
                        continue;

                    }
                } else {
                    if (selectColumn.getTableName().equals("t_basic")) {
                        tBasicMap.put(selectColumn.getColumnName(), value);
                    }
                    if (selectColumn.getTableName().equals("t_basic_sub")) {
                        tBasicSubMap.put(selectColumn.getColumnName(), value);
                    }
                }
            }
            //处理业务
            TBasic tBasic;
            if (tBasicMap != null && tBasicMap.size() > 0) {
                tBasic = JSON.parseObject(JSON.toJSONString(tBasicMap), TBasic.class);
                if (StringUtils.isBlank(tBasic.getEmpId())) {
                    insertLog(tbasic);
                    return ResultUtils.fail("emp_id字段不能为空值");
                }
                tBasic.setUpdateTime(new Date());
                try {
                    itBasicService.updateAndSub(tBasic, tBasicSubMap);
                } catch (RuntimeException e) {
                    insertLog(tbasic);
                    return ResultUtils.fail(e.getMessage());
                }
            } else {
                insertLog(tbasic);
                return ResultUtils.fail("请提供完整的xml");
            }
        } catch (Exception e) {
            e.printStackTrace();
            return ResultUtils.fail();
        }
        return ResultUtils.success().asXML();
    }
}
